# -*- perl -*-

#
# $Id: BBBikeDraw.pm,v 3.3 2001/03/31 21:43:40 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 1998-2001 Slaven Rezic. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: eserte@cs.tu-berlin.de
# WWW:  http://user.cs.tu-berlin.de/~eserte/
#

package BBBikeDraw;
use strict;
use Strassen;
# Strassen benutzt FindBin benutzt Carp, also brauchen wir hier nicht zu
# sparen:
use Carp qw(confess);

use vars qw($images_dir $VERSION);

$VERSION = sprintf("%d.%02d", q$Revision: 3.3 $ =~ /(\d+)\.(\d+)/);

sub new {
    my($pkg, %args) = @_;

    my $self = {};
    $self->{Fh}        = delete $args{Fh} || \*STDOUT;
    $self->{Geometry}  = delete $args{Geometry};
    $self->{Coords}    = delete $args{Coords};
    $self->{Draw}      = delete $args{Draw};
    $self->{Scope}     = delete $args{Scope} || 'city';
    $self->{Startname} = delete $args{Startname};
    $self->{Zielname}  = delete $args{Zielname};
    $self->{Strassen}  = delete $args{Strassen};
    $self->{Wind}      = delete $args{Wind};
    $self->{NoScale}   = delete $args{NoScale};
    $self->{Bg}        = delete $args{Bg}; # "white"/"#rrggbb" . "transparent"
    $self->{UseFlags}  = delete $args{UseFlags};
    $self->{Width}     = delete $args{Width}; # boolean
    $self->{RouteWidth}= delete $args{RouteWidth}; # width of route
    $self->{StrLabel}  = delete $args{StrLabel};
    $self->{MakeNet}   = delete $args{MakeNet};
    $self->{ImageType} = delete $args{ImageType}; # gif, png or wbmp
    $self->{Restrict}  = delete $args{Restrict};  # restriction array
    $self->{GDImage}   = delete $args{GDImage};   # provide old GD image object
    $self->{FrontierColor} = delete $args{FrontierColor}; # "red"

    if ($self->{ImageType} =~ /^pdf$/) {
	$pkg = "BBBikeDraw::PDF";
    } else {
	$pkg = "BBBikeDraw::GD";
    }
    eval "require $pkg"; die $@ if $@;

    bless $self, $pkg;

    $self->init;
}

sub new_from_cgi {
    my($pkg, $q, %args) = @_;
    $args{Geometry}  = $q->param('geometry')
      if defined $q->param('geometry');
    $args{Coords}    = [ split(/[!;]/, $q->param('coords')) ]
      if defined $q->param('coords');
    $args{Draw}      = [ $q->param('draw') ]
      if defined $q->param('draw');
    $args{Scope}     = $q->param('scope')
      if defined $q->param('scope');
    $args{Startname} = $q->param('startname')
      if defined $q->param('startname');
    $args{Zielname} = $q->param('zielname')
      if defined $q->param('zielname');
    if (defined $q->param('windrichtung') &&
	defined $q->param('windstaerke')) {
	$args{Wind} = {Windrichtung => $q->param('windrichtung'),
		       Windstaerke  => $q->param('windstaerke')};
    }
    if (defined $q->param('outputtarget') and
	$q->param('outputtarget') eq 'print') {
	$args{Bg} = 'white';
    }
    $args{UseFlags} = 1;
    # Mit Width (drawwidth) werden breitere Straen gezeichnet. Damit
    # verlangsamt sich auf meinem Celeron 466 das Zeichnen von
    # ca. 3.5 auf 4 Sekunden.
    $args{Width}    = $q->param('drawwidth')
	if defined $q->param('drawwidth');
    $args{StrLabel} = [ $q->param('strlabel') ]
	if defined $q->param('strlabel');
    $args{ImageType} = $q->param('imagetype')
	if defined $q->param('imagetype');
    $pkg->new(%args);
}

sub pre_draw {
    my $self = shift;
    $self->dimension_from_route;
    $self->create_transpose;
}

sub dimension_from_route {
    my $self = shift;
    my(@coords) = @{ $self->{Coords} };
    my @c1;
    my($min_x, $min_y, $max_x, $max_y);
    foreach (@coords) {
	my($x, $y) = split(/,/, $_);
	if (!defined $min_x || $x < $min_x) {
	    $min_x = $x;
	}
	if (!defined $max_x || $x > $max_x) {
	    $max_x = $x;
	}
	if (!defined $min_y || $y < $min_y) {
	    $min_y = $y;
	}
	if (!defined $max_y || $y > $max_y) {
	    $max_y = $y;
	}
	push @c1, [$x,$y];
    }

    if ((!defined $max_x && !defined $min_x) ||
	$max_x == $min_x || $max_y == $min_y) {
	$self->empty_image_error;
    }

    # etwas Luft lassen
    $min_x -= int(0.06*($max_x-$min_x));
    $min_y -= int(0.06*($max_y-$min_y));
    $max_x += int(0.06*($max_x-$min_x));
    $max_y += int(0.06*($max_y-$min_y));

    $self->{Min_x} = $min_x;
    $self->{Min_y} = $min_y;
    $self->{Max_x} = $max_x;
    $self->{Max_y} = $max_y;
    $self->{C1}    = \@c1;
}

sub set_dimension {
    my $self = shift;
    ($self->{Min_x}, $self->{Max_x}, $self->{Min_y}, $self->{Max_y}) = @_;
}

# Setzt die Dimension so, dass die Koordinaten des Strassen-Objekts
# komplett gezeichnet werden knnen.
sub set_dimension_max {
    my($self, $str) = @_;
    $str->init;
    my($min_x, $min_y, $max_x, $max_y);
    while(1) {
	my $s = $str->next_obj;
	last if $s->is_empty;
	for(my $i = 0; $i <= $#{$s->coords}; $i++) {
	    my($x, $y) = @{$s->coord_as_list($i)};
	    if (!defined $min_x || $x < $min_x) {
		$min_x = $x;
	    }
	    if (!defined $max_x || $x > $max_x) {
		$max_x = $x;
	    }
	    if (!defined $min_y || $y < $min_y) {
		$min_y = $y;
	    }
	    if (!defined $max_y || $y > $max_y) {
		$max_y = $y;
	    }
	}
    }
    $self->set_dimension($min_x, $max_x, $min_y, $max_y);
}

sub create_transpose {
    my $self = shift;
    my($w, $h) = ($self->{Width}, $self->{Height});
    my($min_x, $min_y, $max_x, $max_y) =
      ($self->{Min_x}, $self->{Min_y}, $self->{Max_x}, $self->{Max_y});
    my($xk, $yk) = ($w/($max_x-$min_x), $h/($max_y-$min_y));
    my $aspect = ($max_x-$min_x)/($max_y-$min_y);
    my($delta_x, $delta_y) = ($min_x, $min_y);
    if ($aspect < $w/$h) {
	$xk *= $aspect/($w/$h);
	$delta_x -= ($w/$xk-$max_x+$min_x)/2;
    } else {
	$yk /= $aspect/($w/$h);
	$delta_y -= ($h/$yk-$max_y+$min_y)/2;
    }

    my($transpose, $anti_transpose);

    if ($self->isa("BBBikeDraw::PDF")) {
	# Ursprung ist unten, nicht oben
	$transpose = sub {
	    my($x, $y) = @_;
	    ($x-$delta_x)*$xk, ($y-$delta_y)*$yk;
	};
	$anti_transpose = sub {
	    my($x, $y) = @_;
	    (($x/$xk)+$delta_x, (-$y)/$yk+$delta_y); # XXX y???
	};
    } else {
	$transpose = sub {
	    my($x, $y) = @_;
	    (($x-$delta_x)*$xk, $h-($y-$delta_y)*$yk);
	};
	$anti_transpose = sub {
	    my($x, $y) = @_;
	    (($x/$xk)+$delta_x, ($h-$y)/$yk+$delta_y);
	};
    }

    $self->{Transpose}     = $transpose;
    $self->{AntiTranspose} = $anti_transpose;

    $self->{Xk} = $xk;
    $self->{Yk} = $yk;
}

sub set_category_colors {
    my $self = shift;
    my $pkg = ref $self;
    eval "package $pkg;\n" . <<'EOF';

    %color = (B  => $red,
	      H  => $yellow,
	      HH => $yellow,
	      N  => $white,
	      NN => $green,
	      S  => $darkgreen,
	      SA => $darkgreen,
	      SB => $darkgreen,
	      SC => $darkgreen,
	      R  => $darkgreen,
	      RA => $darkgreen,
	      RB => $darkgreen,
	      RC => $darkgreen,
	      U  => $darkblue,
	      UA => $darkblue,
	      UB => $darkblue,
	      W  => $lightblue,
	      W1 => $lightblue,
	      W2 => $lightblue,
	      F  => $white,
	      Ae => $white,
	      P  => $middlegreen,
	      Z  => $self->{FrontierColor} eq 'red' ? $red : $black,
	      Route => $red,
	     );
EOF
    die $@ if $@;
}

sub set_category_widths {
    my $self = shift;
    my $pkg = ref $self;
    eval "package $pkg;\n" . <<'EOF';

    %width = (B  => 3,
	      H  => 3,
	      HH => 3,
	      N  => 2,
	      NN => 2,
	      S  => 2,
	      SA => 2,
	      SB => 2,
	      SC => 2,
	      R  => 2,
	      RA => 2,
	      RB => 2,
	      RC => 2,
	      U  => 2,
	      UA => 2,
	      UB => 2,
	      W  => 2,
	      W1 => 3,
	      W2 => 2,
	      Z  => 1,
	      Route => 3,
	     );
EOF
    die $@ if $@;
}

sub _get_strassen {
    my($self, %args) = @_;
    my $multistr = $self->{Strassen};
    if (!defined $multistr) {
	my $str = new Strassen "strassen";
	if ($self->{Scope} =~ /^(wide)?region$/) {
	    my @s = ($str);
	    push @s, new Strassen "landstrassen";
	    if ($self->{Scope} eq 'wideregion') {
		push @s, new Strassen "landstrassen2";
	    }
	    $multistr = new MultiStrassen @s;
	} else {
	    $multistr = $str;
	}
    }
    $multistr;
}

sub _get_gewaesser {
    my($self, %args) = @_;
    my $multistr;
    my $wstr = new Strassen "wasserstrassen";
    if ($self->{Scope} =~ /^(wide)?region$/) {
	my @w = ($wstr);
	push @w, new Strassen "wasserumland";
	if ($self->{Scope} eq 'wideregion') {
	    push @w, new Strassen "wasserumland2";
	}
	$multistr = new MultiStrassen @w;
    } else {
	$multistr = $wstr;
    }
    $multistr;
}

sub _get_bahn {
    my($self, %args) = @_;
    my $multistr;
    my $bstr = new Strassen "sbahn";
    if ($self->{Scope} =~ /^(wide)?region$/) {
	my $rbstr = new Strassen "rbahn";
	$multistr = new MultiStrassen $bstr, $rbstr;
    } else {
	$multistr = $bstr;
    }
    $multistr;
}

sub _get_orte {
    my($self, %args) = @_;
    my $multistr;
    my $ostr = new Strassen "orte";
    if ($self->{Scope} eq 'wideregion') {
	my @o = ($ostr);
	push @o, new Strassen "orte2";
	$multistr = new MultiStrassen @o;
    } else {
	$multistr = $ostr;
    }
    $multistr;
}

sub suffix {
    shift->{ImageType};
}

sub imagetype {
    shift->{ImageType};
}

sub mimetype {
    my $self = shift;
    ($self->{ImageType} =~ /^pdf$/ ? "application" : "image") . "/"
	. $self->{ImageType};
}

sub get_images_dir {
    if (!defined $images_dir) {
	require File::Basename;
	$images_dir = $INC{'BBBikeDraw.pm'};
	if ($images_dir ne '') {
	    $images_dir = File::Basename::dirname($images_dir);
	} else {
	    $images_dir = "/home/e/eserte/src/bbbike";
	}
	$images_dir .= "/images";
    }
    $images_dir;
}

1;
