# -*- perl -*-

#
# $Id: G7toWin_ASCII.pm,v 1.4 2001/03/24 22:06:10 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2001 Slaven Rezic. All rights reserved.
# This package is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: eserte@cs.tu-berlin.de
# WWW:  http://user.cs.tu-berlin.de/~eserte/
#

package GPS::G7toWin_ASCII;
use base qw(GPS);

use strict;

sub magics { ('^#', '^D\s+WGS-84', '^M\s+DMM') }

sub convert_to_route {
    my($self, $file, %args) = @_;

    my($fh, $lines_ref) = $self->overread_trash($file, %args);
    die "File $file does not match" unless $fh;

    require Karte::Polar;
    my $obj = $Karte::Polar::obj;

    my @res;
    my $check = sub {
	my $line = shift;
	chomp;
	if (m|^W\s+(?:\w+)\s+([NS])(\d+)\s+([\d.]+)\s+([EW])(\d+)\s+([\d.]+)|) {
	    my $breite = $2;
	    my $laenge = $5;

	    my $breite_min = $3/60;
	    my $laenge_min = $6/60;

	    $breite += $breite_min;
	    $laenge += $laenge_min;

	    if ($1 eq 'S') { $breite = -$breite }
	    if ($3 eq 'W') { $laenge = -$laenge }
	    my($x,$y) = $obj->map2standard($laenge, $breite);
	    if (!@res || ($x != $res[-1]->[0] ||
			  $y != $res[-1]->[1])) {
		push @res, [$x, $y];
	    }
	}
    };

    $check->($_) foreach @$lines_ref;
    while(<$fh>) {
	$check->($_);
    }

    close $fh;

    @res;
}

sub convert_from_route {
    my($self, $route, %args) = @_;

    require Karte::Polar;
    my $obj = $Karte::Polar::obj;

    my $routename = sprintf("%-8s", $args{-routename} || "TRACBACK");
    my $str       = $args{-streetobj};
    my %crossings;
    if ($str) {
	%crossings = $str->all_crossings(RetType => 'hash',
					 UseCache => 1);
    }

    my $s = <<EOF;
#@{[ scalar localtime ]}\015
D WGS-84\015
M DMM\015
R  20 $routename\015
EOF
    my @path = $route->path_list;
    my $point_number = $#path;
    foreach my $xy (@path) {
	my $xy_string = join ",", @$xy;
	my($polar_x, $polar_y) = $obj->standard2map(@$xy);
	my $NS = $polar_y > 0 ? "N" : do { $polar_y = -$polar_y; "S" };
	my $EW = $polar_x > 0 ? "E" : do { $polar_x = -$polar_x; "W" };
	my $ns_deg = int($polar_y);
	my $ew_deg = int($polar_x);
	my $ns_min = ($polar_y-$ns_deg)*60;
	my $ew_min = ($polar_x-$ew_deg)*60;

	my $comment = "";
	if ($str && exists $crossings{$xy_string}) {
	    $comment = join("/", map { s/\s+\(.*\)\s*$//; $_ } @{ $crossings{$xy_string} });
	    if (length($comment) > 45) {
		$comment = substr($comment, 0, 45);
	    }
	}

	$s .= sprintf
	    "%-3s %s%03d            %s%02s %07.4f %s%03d %07.4f %-45s; 8196;1;0\015\012",
	    "W",
	    "T", $point_number,
	    $NS, $ns_deg, $ns_min,
	    $EW, $ew_deg, $ew_min,
	    $comment,
	    ;
	$point_number--;
    }
    $s .= "E  20\015\012";
    $s;
}

1;

__END__
