# -*- perl -*-
# $Id: Makefile.PL,v 1.55 2001/04/07 15:22:22 eserte Exp eserte $

BEGIN {
    $^W = 0;
}

use ExtUtils::MakeMaker;
use ExtUtils::Manifest;
use File::Basename;

use lib ".."; # falls als make_bundle gestartet

use BBBikeVar;

use constant REQTABLE_MODNAME => 0;
use constant REQTABLE_VERSION => 1;
use constant REQTABLE_OS      => 2;
use constant REQTABLE_REQ     => 3;
use constant REQTABLE_DESC    => 4;
use constant REQTABLE_LONGDESC=> 5;

# Table of prerequisites:
# Name, Version, OS, Req/CoReq, description token, long description
@req_table =
 ([qw/Tk		402.002	all	1	gui/,
   'das absolute Mu!'],
  #in dist: [qw/Tk::Getopt	0.40	all	0	gui/],
  #in dist: [qw/Tk::ContextHelp	0	unix	0	gui/],
  [qw/Tk::FireButton	0	all	0	gui/],
  [qw/Tk::Pod		2.8	all	0	gui/,
   'Online-Hilfe'],
  [qw/Tk::FontDialog	0	all	0	gui/],
  [qw/Tk::JPEG		0	all	0	gui/],
#  [qw/X11::Protocol	0.04	unix	0	gui/], # not used anymore
  [qw/Tie::Watch	0	all	0	gui/],
  [qw/Tk::HistEntry	0	all	0	gui/],

  [qw/LWP::UserAgent	0	all	0	www/,
   'fr die WWW-Verbindungen (Stadtplan und Wetterbericht)'],
  [qw/Image::Magick	0	all	0	www/,
   'fr Bildmanipulationen beim Radar-Bild der FU'],

  [qw/CGI::MiniSvr	0	all	0	cgi/,
   'fr das persistente CGI-Programm'],
  [qw/FCGI		0	all	0	cgi/],

  [qw/Mail::Send	0	unix	0	mail/,
   'falls man aus bbbike heraus E-Mails mit der Routenbeschreibung verschicken will'],

  [qw/Win32::Sound	0	win	0	audio/],
  [qw/Audio::Data	0.003	unix	0	audio/],

  [qw/String::Approx	2.7	all	0	base/,
   'oder man verwendet agrep (mindestens Version 3.0)'],
  [qw/Storable		0	all	0	base/,
   'fr das Caching beim CGI-Programm'],
  [qw/MLDBM		0	all	0	base/],
  [qw/GD		1.18	all	0	base/,
   'zum On-the-fly-Erzeugen von Grafiken beim CGI-Programm'],
  [qw/List::Permutor	0	all	0	base/,
   'Fr das Problem des Handlungsreisenden'],
  [qw/PDF::Create	0	all	0	base/,
   'Erzeugung der Route als PDF-Dokument'],
  #in dist: [qw/BikePower	0	all	0	base/],

  [qw/BSD::Resource	0	unix	0	base/], # really only BSD

  [qw/Win32::API	0	win	0	base/],
  [qw/Win32::Registry	0	win	0	base/],
  [qw/Win32::DDE::Client 0	win	0	base/],
  [qw/Win32::Shortcut	0	win	0	base/],

  [qw/Devel::Peek	0	all	0	devel/],
  [qw/Statistics::Descriptive	0	all	0	devel/],
  [qw/Math::MatrixReal	0	all	0	devel/],

 );

# EXE prerequisites ####################################################
# standard Unix: uname, who, tee, at, diff, zcat, ping
# BSD Unix:      leave
# extra:         enscript, gv, ghostview, gs, netpbm-utils, cjpeg, agrep,
#                netscape, socket
# local extra:   cdmount, dial
# palm utils:    install-datebook, pilot_makedoc, iSilo*, txt2pdbdoc,
#                pilot-xfer
# standard X11:  xwd
########################################################################

# build prereq for the current os
my $os; # XXX os2, msdos, mac et al. missing
if ($^O eq 'MSWin32') {
    $os = "win";
} else {
    $os = "unix";
}

my(%prereq, @prereq);
foreach (@req_table) {
    my(%for_os) = map { ($_, 1) } split ",", $_->[REQTABLE_OS];
    if ($for_os{$os} || $for_os{"all"}) {
	$prereq{$_->[REQTABLE_MODNAME]} = $_->[REQTABLE_VERSION];
	push @prereq, $_;
    }
}

# call before maniread!
exit make_bundle()       if $0 =~ /make_bundle/;
exit make_small_bundle() if $0 =~ /make_small_bundle/;

$fileref = ExtUtils::Manifest::maniread();
$exe_files = ['bbbike',
	      'bbbikeapplet',
	      'bbbikeclient',
	      'cgi/bbbike.cgi',
	      'cgi/runbbbikecgi',
	      'cgi/httpi',
	      'cgi/wapbbbike.cgi',
	      'cmdbbbike',
	      'smsbbbike',
	      'cbbbike'];
foreach (@$exe_files) { $exe_files{$_}++ }
while(my($k,$v) = each %$fileref) {
    next if $k =~ /\.bat$/;
    $pm{$k} = '$(INST_LIB)/'. $k unless $exe_files{$k};
}

return 1 if $ENV{PERL_FROM_EXTERNAL};

sub make_bundle {
    make_general_bundle(0, "BBBike.pm");
}

sub make_small_bundle {
    make_general_bundle(1, "BBBike_small.pm");
}

sub make_general_bundle {
    my $type = shift;
    my $file = shift;
    open(TMPL, "BBBike.pm.tmpl") or die "Can't open template: $!";
    open(BUNDLE, ">$file") or die "Can't write bundle $file: $!";
    while(<TMPL>) {
	print BUNDLE $_;
	if (/^=head1\s+CONTENTS/) {
	    print BUNDLE "\n";
	    foreach my $prereq (@prereq) {
		next if $prereq->[REQTABLE_REQ] < $type;
		print BUNDLE $prereq->[REQTABLE_MODNAME];
		if ($prereq->[REQTABLE_VERSION]) {
		    print BUNDLE " " . $prereq->[REQTABLE_VERSION];
		}
		if (defined $prereq->[REQTABLE_LONGDESC]) {
		    print BUNDLE "\t- " . $prereq->[REQTABLE_LONGDESC];
		}
		print BUNDLE "\n\n";
	    }
	}
    }
    close BUNDLE;
    close TMPL;
    0;
}

WriteMakefile
  (
   'NAME'	  => 'BBBike',
   'VERSION_FROM' => 'BBBikeVar.pm',
   'dist'         => {'COMPRESS'=>'gzip -9f', 'SUFFIX' => '.gz',
		      'POSTOP'=>'-$(CHMOD) 644 $(DISTVNAME).tar$(SUFFIX)'},
   'PM'           => \%pm,
   'PREREQ_PM'    => \%prereq,
   'EXE_FILES'    => $exe_files,

   'ABSTRACT'     => "BBBike - ein Routenplaner fr Radfahrer in Berlin und Brandenburg",
   'AUTHOR'       => 'Slaven Rezic (' . $BBBike::EMAIL . ')',

   'INSTALLER'    =>
   { CONSOLE_PROGRAM        => "cbbbike",
     CLIENT_PROGRAM         => "bbbikeclient",
     USE_CLIENT_SERVER_MODE => 0,
     USE_AUTOSTART          => undef,
     HTML_DOCUMENTATION     => "bbbike.html",
     HOMEPAGE               => $BBBike::BBBIKE_WWW,
     MODULE_EXT             => ['BBBikeXS', ($^O eq 'MSWin32' ? () : ('VirtArray'))],
     EXTENSIONS             => ($0 eq '/usr/local/BBBike/install.pl' &&
				$^O =~ /((free|net|open)bsd|bsdi)$/ ? 0 : 1),
     DESKTOP_ICON           => 1,
     UNIX_ICON              => "images/srtbike.xpm",
     UNIX_MINI_ICON         => "images/srtbike_mini.xpm",
     WIN_ICON               => "images/srtbike.ico",
     WIN_WWW_ICON           => "images/srtbike_www.ico",
     WIN_EXTENSIONS         => [{'-extension' => '.bbr',
				 '-name'      => 'BBBike.Route',
				 '-desc'      => 'BBBike-Route',
				 '-mime'      => "application/x-bbbike-route",
				}],
   },
  );

sub MY::top_targets {
    'all ::
	@echo "NYI"

'
}

sub MY::install {
    <<'EOF';
install:
	@echo "*** This will proceed an unusual installation process."
	@echo "*** The program and data will reside in this directory."
	@echo "*** The installer will fetch dependent modules and"
	@echo "*** try to make some GUI installation (currently for KDE, GNOME and Win32"
	@echo ""
	@sleep 1
	-$(PERL) -MCPAN -e "install Bundle::BBBike"
	$(PERL) install.pl
#	@echo "Copy this directory to an appropriate destination (e.g. /usr/local/BBBike)"
#	@echo "and create a symbolic link to the executable"
#	@echo "(e.g. from /usr/local/bin/bbbike to /usr/local/BBBike/bbbike)"

EOF
}

sub MY::postamble {
    <<'EOF' .

# DISTRIBUTIONS ##########################################################

TMPDIR?=	/tmp
BBBIKE_TMP=     $(TMPDIR)/$(DISTNAME)

EOF

    "DEVEL_ARCH=       \$(TMPDIR)/\$(DISTVNAME)-" .
                        sprintf("%04d%02d%02d",
                               sub { $_[5]+1900, $_[4]+1, $_[3] }->(localtime)
                              ) . "-devel.zip\n\n" .

    <<'EOF' .
# bbbike.cgi #####################

CGI_ARCH=	$(TMPDIR)/bbbikecgi.zip
CGI2_ARCH_PROG= $(TMPDIR)/bbbikecgi_prog.zip
CGI2_ARCH_WEB=  $(TMPDIR)/bbbikecgi_web.zip

# CGI-Distribution: alle Dateien in einer Hierarchie
cgidist: cgidistdir
	$(RM_F) $(CGI_ARCH)
	cd $(BBBIKE_TMP); $(ZIP) $(ZIPFLAGS) $(CGI_ARCH) $(DISTNAME)

cgidistdir:   MANIFEST.bbbikecgi
	$(RM_RF) $(BBBIKE_TMP)
	$(MKPATH) $(BBBIKE_TMP)
	$(PERL) -I$(PERL_ARCHLIB) -I$(PERL_LIB) \
		-MExtUtils::Manifest=manicopy,maniread \
                -e "manicopy(maniread(q{MANIFEST.bbbikecgi}),q{$(BBBIKE_TMP)/$(DISTNAME)}, q{$(DIST_CP)});"

# Alternative: Programme unter cgi-bin, Rest in ein BBBike-Verzeichnis
cgidist2:
	$(RM_F) $(CGI2_ARCH_PROG) $(CGI2_ARCH_WEB)
	$(RM_RF) $(BBBIKE_TMP)
	$(MKPATH) $(BBBIKE_TMP)
	$(PERL) -I$(PERL_ARCHLIB) -I$(PERL_LIB) \
		-MExtUtils::Manifest=manicopy,maniread \
                -e "manicopy(maniread(q{MANIFEST.bbbikecgi.prog}),q{$(BBBIKE_TMP)/$(DISTNAME)}, q{$(DIST_CP)});"
	$(MV) $(BBBIKE_TMP)/$(DISTNAME)/cgi/* $(BBBIKE_TMP)
	cd $(BBBIKE_TMP); $(ZIP) $(ZIPFLAGS) $(CGI2_ARCH_PROG) .
	$(RM_RF) $(BBBIKE_TMP)
	$(MKPATH) $(BBBIKE_TMP)
	$(PERL) -I$(PERL_ARCHLIB) -I$(PERL_LIB) \
		-MExtUtils::Manifest=manicopy,maniread \
                -e "manicopy(maniread(q{MANIFEST.bbbikecgi.web}),q{$(BBBIKE_TMP)/$(DISTNAME)}, q{$(DIST_CP)});"
	cd $(BBBIKE_TMP); $(ZIP) $(ZIPFLAGS) $(CGI2_ARCH_WEB) $(DISTNAME)

# In beiden MANIFEST-Dateien sind images enthalten, weil images sowohl vom
# Browser aus, als auch bei der PNG/GIF-Erstellung verwendet werden.
MANIFEST.bbbikecgi:	MANIFEST.bbbikecgi.prog MANIFEST.bbbikecgi.web
	echo "# DO NOT EDIT! Generated by Makefile" > MANIFEST.bbbikecgi
	cat MANIFEST.bbbikecgi.prog MANIFEST.bbbikecgi.web >> MANIFEST.bbbikecgi

# cbbbike #####################

CBBBIKE_ARCH=      /tmp/cbbbike.zip
CBBBIKE_DOS_ARCH=  /tmp/cbbbike_dos.zip
RECODE=            recode

.PHONY: ChangeLog develdist makepatch cbbbikedist cbbbikedistdit
.PHONY: cbbbikedosdist cbbbikedosdistdir cgidist cgidistdir
.PHONY: ext

cbbbikedist: cbbbikedistdir
	$(RM_F) $(CBBBIKE_ARCH)
	cd /tmp; $(ZIP) $(ZIPFLAGS) $(CBBBIKE_ARCH) $(DISTNAME)

cbbbikedistdir:
	$(RM_RF) $(BBBIKE_TMP)
	$(PERL) -I$(PERL_ARCHLIB) -I$(PERL_LIB) -MExtUtils::Manifest=manicopy,maniread \
                -e "manicopy(maniread(\'MANIFEST.cbbbike\'),\'$(BBBIKE_TMP)\', \'$(DIST_CP)\');"

# cbbbikedosdist: wie cbbbikedist, nur wird noch zustzlich mit recode
# nach cp850 (besser als cp437) konvertiert
cbbbikedosdist: cbbbikedosdistdir
	$(RM_F) $(CBBBIKE_DOS_ARCH)
	cd /tmp; $(ZIP) $(ZIPFLAGS) $(CBBBIKE_DOS_ARCH) $(DISTNAME)

cbbbikedosdistdir: cbbbikedistdir
	cd $(BBBIKE_TMP); chmod u+w *; $(RECODE) latin1:cp850 *.pm cbbbike; chmod 755 cbbbike
	cd $(BBBIKE_TMP)/data; chmod u+w *; $(RECODE) latin1:cp850 *

develdist:
	rm -rf $(DEVEL_ARCH)
	cd ..; $(ZIP) $(ZIPFLAGS) $(DEVEL_ARCH) bbbike \
	    -x \*.tar.gz \
	    -x \*/old/\* \
	    -x \*/cache/\* \
	    -x \*/tmp/\* \
	    -x \*/vbbdata/\* \
	    -x \*/c/minibbbike \
	    -x \*/datagdf_berlin/\* \
	    -x \*/datagdf_brandenburg/\* \
	    -x \*/datagdf_mitte/\* \
	    -x \*/datagdf_*/\* \
	    -x \*/datagdf/\* \
	    -x \*/diplom/\* \
	    -x \*/CRASH/\* \
	    -x \*/distfiles/\* \
	    -x \*/i386-freebsd/\* \
            -x \*/i386-freebsd-64int/\* \
            -x \*/i386-freebsd-thread/\* \
	    -x \*/i586-linux/\* \
	    -x \*/blib \
	    -x \*/blib/\* \
	    -x \*/pm_to_blib \
 	    -x \*/.xvpics/\* \
	    -x \*/misc/adressen/\* \
	    -x \*/misc/Berlin_Mitte.gdf \
	    -x \*/palm/test/\* \
	    -x \*/palm/test2/\* \
	    -x \*/projects/\* \
	    -x \*/vbbbike/vbbbike \
	    -x \*.o \
	    -x \*.so \
	    -x \*.bs \
	    -x \*.old \
	    -x \*.core \
	    -x \*.out \
	    -x \*.grc \
	    -x \*.bin \
	    -x \*/#*# \
	    -x \*.gdbinit \
	    -x \*.bak \
	    -x \*~

win32sdist:
	./miscsrc/win32sdist

predist:
	cd data; ${MAKE}
	${MAKE} extmod
	${MAKEl linkextmod
	${MAKE} distcheck
	${MAKE} distdir
	if [ ! -h Bundle/make_bundle ] ; then \
	    ln -s ../Makefile.PL Bundle/make_bundle; \
	fi
	if [ ! -h Bundle/make_small_bundle ] ; then \
	    ln -s ../Makefile.PL Bundle/make_small_bundle; \
	fi
EOF
"	cd Bundle; $^X make_bundle
	cd Bundle; $^X make_small_bundle
" . <<'EOF'
	@echo check bbbike in BBBike-x.xx directory
	@echo Versionsnummer in README.spp ndern

# DOCUMENTATION ##########################################################

CLEAN_POD2HTML=$(RM_F) pod2html-*cache pod2htm?.?~~

# XXX --flush works not like documented?
bbbike.html:	bbbike.pod
	/usr/perl5.005_670d/bin/pod2html bbbike.pod > bbbike.html
	$(CLEAN_POD2HTML)

STRPP=	strpp

README README.english:	README.spp
	${STRPP} --dir=. README.spp

README.html:	README
	pod2html README > README.html
	chmod 644 README.html
	$(CLEAN_POD2HTML)

README.en.html:	README.english
	pod2html README.english > README.en.html
	chmod 644 README.en.html
	$(CLEAN_POD2HTML)

clean_pod2html:

bbbikedoc:	README README.english README.html README.en.html bbbike.html

# XXX this will fail if there is no symlink to the original libexec position
ChangeLog:
	rcs2log -R > ChangeLog

clean_cache:
	echo rm -rf cache/*

# PORTS, BINDISTS...####################################################

fbsdport:
	cd port/freebsd; ./mkport.pl

WINBINDISTDIR=/tmp/BBBike-$(VERSION)-Windows
WINBINDIST=$(WINBINDISTDIR).zip

winbindist:
	rm -rf $(WINBINDISTDIR)
	mkdir $(WINBINDISTDIR)
	cp -Rp $(HOME)/src/binperl/windows $(WINBINDISTDIR)
	cat distfiles/BBBike-$(VERSION).tar.gz | (cd $(WINBINDISTDIR); tar xfvz - )
	mv $(WINBINDISTDIR)/BBBike-$(VERSION) $(WINBINDISTDIR)/bbbike
	cp -p cdrom/windows/autorun.inf \
	      cdrom/windows/bbbike.bat \
	      cdrom/windows/setup.bat \
	      cdrom/windows/setup.pl \
	    $(WINBINDISTDIR)
	cd $(WINBINDISTDIR)/..; umask 022; zip -r $(WINBINDIST) `basename $(WINBINDISTDIR)`

########################################################################

alldist: bbbikedoc dist dist_mv makepatch fbsdport winbindist ChangeLog

dist_mv:
	mv -i BBBike-$(VERSION).tar.gz distfiles 

BACKUP_FILE=	/home5/bak/bbbike-devel.tar.gz 

backup:
	-[ -f ${BACKUP_FILE} ] && mv -f ${BACKUP_FILE} ${BACKUP_FILE}~
	cd ..; tar -c -v -z -f ${BACKUP_FILE} --exclude "*.tar.gz" --exclude "*patch.gz" --exclude "*~" bbbike

# XXX Problem: binary files (xdelta? uuencode? nix?)
makepatch:
#	./miscsrc/mymakepatch.pl $(NAME)

rcslabel:
	$(PERL) -I$(PERL_ARCHLIB) -I$(PERL_LIB) -MExtUtils::Manifest=maniread \
		-e "@all = keys %{ maniread() };" \
		-e 'print("Executing $(RCS_LABEL) ...\n"); system("$(RCS_LABEL) @all");'

autoload:
	./miscsrc/make_autoload.pl -dd -f


# Pfad zum mymake-Skript, das Makefile.PL's fr ~/lib/perl erzeugt
# Da vieler meiner Skripte Test.pm verwenden, wird das nicht mit 5.004 gehen
EOF
		    . "MYEXTMODMAKE=	\$(HOME)/src/mymake -perl $^X"
		    . <<'EOF'
# normales Erzeugen mit: $(FULLPERL) Makefile.PL
# "offizielle" Module
EXTMODCPANTOP=	$(HOME)/src/CPAN
EXTMODCPAN=	BikePower Tk-ContextHelp Tk-Getopt savevars
# "experimentelle" Module
EXTMODPERLTOP=	$(HOME)/src/perl
EXTMODPERL=	Http Tk-SREZIC Tk-CanvasBalloon Tk-Enscript Tk-LogScale
EXTMODPERL+=	Tk-Splash Tk-StippleLine Tk-WidgetDump Win32Util WWWBrowser

echoextmodcpantop:
	@echo $(EXTMODCPANTOP)

echoextmodcpan:
	@echo $(EXTMODCPAN)

echoextmodperltop:
	@echo $(EXTMODPERLTOP)

echoextmodperl:
	@echo $(EXTMODPERL)

# baut und installiert die externen Module wie BikePower etc.
extmod:
	for i in $(EXTMODCPAN); do \
	    echo $$i; \
	    cd $(EXTMODCPANTOP)/$$i; \
		$(MYEXTMODMAKE) && \
		make && \
		BATCH=yes make test && \
		make install; \
	done
	for i in $(EXTMODPERL); do \
	    echo $$i; \
	    cd $(EXTMODPERLTOP)/$$i; \
		$(MYEXTMODMAKE) && \
		make && \
		BATCH=yes make test && \
		make install; \
	done

linkextmod:
	./miscsrc/link_ext_mod.pl

ext:
	cd ext; ${MAKE} ext

echoversion:
	@echo 1>&2 This rule is obsolete ... use BBBikeVar.pm
	@echo $(VERSION)
EOF
;
}
